/**
 * more opinionated config.
 * @type {Object}
 */
module.exports = {
  plugins: ['import'],
  rules: {
    'import/no-named-as-default': 1,
    'import/no-named-as-default-member': 1,
    'import/no-duplicates': 1,
  },
}
