/**
 * Rules in progress.
 *
 * Do not expect these to adhere to semver across releases.
 * @type {Object}
 */
module.exports = {
  plugins: ['import'],
  rules: {
    'import/no-deprecated': 1,
  },
}
